// Code generated by astool. DO NOT EDIT.

package streams

import (
	typepropertyvalue "code.superseriousbusiness.org/activity/streams/impl/schema/type_propertyvalue"
	vocab "code.superseriousbusiness.org/activity/streams/vocab"
)

// NewSchemaPropertyValue creates a new SchemaPropertyValue
func NewSchemaPropertyValue() vocab.SchemaPropertyValue {
	return typepropertyvalue.NewSchemaPropertyValue()
}
