// Code generated by astool. DO NOT EDIT.

package streams

import (
	typepropertyvalue "code.superseriousbusiness.org/activity/streams/impl/schema/type_propertyvalue"
	vocab "code.superseriousbusiness.org/activity/streams/vocab"
)

// IsOrExtendsSchemaPropertyValue returns true if the other provided type is the
// PropertyValue type or extends from the PropertyValue type.
func IsOrExtendsSchemaPropertyValue(other vocab.Type) bool {
	return typepropertyvalue.IsOrExtendsPropertyValue(other)
}
