// Code generated by astool. DO NOT EDIT.

package propertycanquote

import vocab "code.superseriousbusiness.org/activity/streams/vocab"

var mgr privateManager

// privateManager abstracts the code-generated manager that provides access to
// concrete implementations.
type privateManager interface {
	// DeserializeCanQuoteGoToSocial returns the deserialization method for
	// the "GoToSocialCanQuote" non-functional property in the vocabulary
	// "GoToSocial"
	DeserializeCanQuoteGoToSocial() func(map[string]interface{}, map[string]string) (vocab.GoToSocialCanQuote, error)
}

// SetManager sets the manager package-global variable. For internal use only, do
// not use as part of Application behavior. Must be called at golang init time.
func SetManager(m privateManager) {
	mgr = m
}
