// Code generated by astool. DO NOT EDIT.

package streams

import (
	propertyaccuracy "code.superseriousbusiness.org/activity/streams/impl/activitystreams/property_accuracy"
	propertyactor "code.superseriousbusiness.org/activity/streams/impl/activitystreams/property_actor"
	propertyalsoknownas "code.superseriousbusiness.org/activity/streams/impl/activitystreams/property_alsoknownas"
	propertyaltitude "code.superseriousbusiness.org/activity/streams/impl/activitystreams/property_altitude"
	propertyanyof "code.superseriousbusiness.org/activity/streams/impl/activitystreams/property_anyof"
	propertyattachment "code.superseriousbusiness.org/activity/streams/impl/activitystreams/property_attachment"
	propertyattributedto "code.superseriousbusiness.org/activity/streams/impl/activitystreams/property_attributedto"
	propertyaudience "code.superseriousbusiness.org/activity/streams/impl/activitystreams/property_audience"
	propertybcc "code.superseriousbusiness.org/activity/streams/impl/activitystreams/property_bcc"
	propertybto "code.superseriousbusiness.org/activity/streams/impl/activitystreams/property_bto"
	propertycc "code.superseriousbusiness.org/activity/streams/impl/activitystreams/property_cc"
	propertyclosed "code.superseriousbusiness.org/activity/streams/impl/activitystreams/property_closed"
	propertycontent "code.superseriousbusiness.org/activity/streams/impl/activitystreams/property_content"
	propertycontext "code.superseriousbusiness.org/activity/streams/impl/activitystreams/property_context"
	propertycurrent "code.superseriousbusiness.org/activity/streams/impl/activitystreams/property_current"
	propertydeleted "code.superseriousbusiness.org/activity/streams/impl/activitystreams/property_deleted"
	propertydescribes "code.superseriousbusiness.org/activity/streams/impl/activitystreams/property_describes"
	propertyduration "code.superseriousbusiness.org/activity/streams/impl/activitystreams/property_duration"
	propertyendpoints "code.superseriousbusiness.org/activity/streams/impl/activitystreams/property_endpoints"
	propertyendtime "code.superseriousbusiness.org/activity/streams/impl/activitystreams/property_endtime"
	propertyfirst "code.superseriousbusiness.org/activity/streams/impl/activitystreams/property_first"
	propertyfollowers "code.superseriousbusiness.org/activity/streams/impl/activitystreams/property_followers"
	propertyfollowing "code.superseriousbusiness.org/activity/streams/impl/activitystreams/property_following"
	propertyformertype "code.superseriousbusiness.org/activity/streams/impl/activitystreams/property_formertype"
	propertygenerator "code.superseriousbusiness.org/activity/streams/impl/activitystreams/property_generator"
	propertyheight "code.superseriousbusiness.org/activity/streams/impl/activitystreams/property_height"
	propertyhref "code.superseriousbusiness.org/activity/streams/impl/activitystreams/property_href"
	propertyhreflang "code.superseriousbusiness.org/activity/streams/impl/activitystreams/property_hreflang"
	propertyicon "code.superseriousbusiness.org/activity/streams/impl/activitystreams/property_icon"
	propertyimage "code.superseriousbusiness.org/activity/streams/impl/activitystreams/property_image"
	propertyinbox "code.superseriousbusiness.org/activity/streams/impl/activitystreams/property_inbox"
	propertyinreplyto "code.superseriousbusiness.org/activity/streams/impl/activitystreams/property_inreplyto"
	propertyinstrument "code.superseriousbusiness.org/activity/streams/impl/activitystreams/property_instrument"
	propertyitems "code.superseriousbusiness.org/activity/streams/impl/activitystreams/property_items"
	propertylast "code.superseriousbusiness.org/activity/streams/impl/activitystreams/property_last"
	propertylatitude "code.superseriousbusiness.org/activity/streams/impl/activitystreams/property_latitude"
	propertyliked "code.superseriousbusiness.org/activity/streams/impl/activitystreams/property_liked"
	propertylikes "code.superseriousbusiness.org/activity/streams/impl/activitystreams/property_likes"
	propertylocation "code.superseriousbusiness.org/activity/streams/impl/activitystreams/property_location"
	propertylongitude "code.superseriousbusiness.org/activity/streams/impl/activitystreams/property_longitude"
	propertymanuallyapprovesfollowers "code.superseriousbusiness.org/activity/streams/impl/activitystreams/property_manuallyapprovesfollowers"
	propertymediatype "code.superseriousbusiness.org/activity/streams/impl/activitystreams/property_mediatype"
	propertymovedto "code.superseriousbusiness.org/activity/streams/impl/activitystreams/property_movedto"
	propertyname "code.superseriousbusiness.org/activity/streams/impl/activitystreams/property_name"
	propertynext "code.superseriousbusiness.org/activity/streams/impl/activitystreams/property_next"
	propertyobject "code.superseriousbusiness.org/activity/streams/impl/activitystreams/property_object"
	propertyoneof "code.superseriousbusiness.org/activity/streams/impl/activitystreams/property_oneof"
	propertyordereditems "code.superseriousbusiness.org/activity/streams/impl/activitystreams/property_ordereditems"
	propertyorigin "code.superseriousbusiness.org/activity/streams/impl/activitystreams/property_origin"
	propertyoutbox "code.superseriousbusiness.org/activity/streams/impl/activitystreams/property_outbox"
	propertypartof "code.superseriousbusiness.org/activity/streams/impl/activitystreams/property_partof"
	propertypreferredusername "code.superseriousbusiness.org/activity/streams/impl/activitystreams/property_preferredusername"
	propertyprev "code.superseriousbusiness.org/activity/streams/impl/activitystreams/property_prev"
	propertypreview "code.superseriousbusiness.org/activity/streams/impl/activitystreams/property_preview"
	propertypublished "code.superseriousbusiness.org/activity/streams/impl/activitystreams/property_published"
	propertyradius "code.superseriousbusiness.org/activity/streams/impl/activitystreams/property_radius"
	propertyrel "code.superseriousbusiness.org/activity/streams/impl/activitystreams/property_rel"
	propertyrelationship "code.superseriousbusiness.org/activity/streams/impl/activitystreams/property_relationship"
	propertyreplies "code.superseriousbusiness.org/activity/streams/impl/activitystreams/property_replies"
	propertyresult "code.superseriousbusiness.org/activity/streams/impl/activitystreams/property_result"
	propertysensitive "code.superseriousbusiness.org/activity/streams/impl/activitystreams/property_sensitive"
	propertysharedinbox "code.superseriousbusiness.org/activity/streams/impl/activitystreams/property_sharedinbox"
	propertyshares "code.superseriousbusiness.org/activity/streams/impl/activitystreams/property_shares"
	propertysource "code.superseriousbusiness.org/activity/streams/impl/activitystreams/property_source"
	propertystartindex "code.superseriousbusiness.org/activity/streams/impl/activitystreams/property_startindex"
	propertystarttime "code.superseriousbusiness.org/activity/streams/impl/activitystreams/property_starttime"
	propertystreams "code.superseriousbusiness.org/activity/streams/impl/activitystreams/property_streams"
	propertysubject "code.superseriousbusiness.org/activity/streams/impl/activitystreams/property_subject"
	propertysummary "code.superseriousbusiness.org/activity/streams/impl/activitystreams/property_summary"
	propertytag "code.superseriousbusiness.org/activity/streams/impl/activitystreams/property_tag"
	propertytarget "code.superseriousbusiness.org/activity/streams/impl/activitystreams/property_target"
	propertyto "code.superseriousbusiness.org/activity/streams/impl/activitystreams/property_to"
	propertytotalitems "code.superseriousbusiness.org/activity/streams/impl/activitystreams/property_totalitems"
	propertyunits "code.superseriousbusiness.org/activity/streams/impl/activitystreams/property_units"
	propertyupdated "code.superseriousbusiness.org/activity/streams/impl/activitystreams/property_updated"
	propertyurl "code.superseriousbusiness.org/activity/streams/impl/activitystreams/property_url"
	propertywidth "code.superseriousbusiness.org/activity/streams/impl/activitystreams/property_width"
	vocab "code.superseriousbusiness.org/activity/streams/vocab"
)

// NewActivityStreamsActivityStreamsAccuracyProperty creates a new
// ActivityStreamsAccuracyProperty
func NewActivityStreamsAccuracyProperty() vocab.ActivityStreamsAccuracyProperty {
	return propertyaccuracy.NewActivityStreamsAccuracyProperty()
}

// NewActivityStreamsActivityStreamsActorProperty creates a new
// ActivityStreamsActorProperty
func NewActivityStreamsActorProperty() vocab.ActivityStreamsActorProperty {
	return propertyactor.NewActivityStreamsActorProperty()
}

// NewActivityStreamsActivityStreamsAlsoKnownAsProperty creates a new
// ActivityStreamsAlsoKnownAsProperty
func NewActivityStreamsAlsoKnownAsProperty() vocab.ActivityStreamsAlsoKnownAsProperty {
	return propertyalsoknownas.NewActivityStreamsAlsoKnownAsProperty()
}

// NewActivityStreamsActivityStreamsAltitudeProperty creates a new
// ActivityStreamsAltitudeProperty
func NewActivityStreamsAltitudeProperty() vocab.ActivityStreamsAltitudeProperty {
	return propertyaltitude.NewActivityStreamsAltitudeProperty()
}

// NewActivityStreamsActivityStreamsAnyOfProperty creates a new
// ActivityStreamsAnyOfProperty
func NewActivityStreamsAnyOfProperty() vocab.ActivityStreamsAnyOfProperty {
	return propertyanyof.NewActivityStreamsAnyOfProperty()
}

// NewActivityStreamsActivityStreamsAttachmentProperty creates a new
// ActivityStreamsAttachmentProperty
func NewActivityStreamsAttachmentProperty() vocab.ActivityStreamsAttachmentProperty {
	return propertyattachment.NewActivityStreamsAttachmentProperty()
}

// NewActivityStreamsActivityStreamsAttributedToProperty creates a new
// ActivityStreamsAttributedToProperty
func NewActivityStreamsAttributedToProperty() vocab.ActivityStreamsAttributedToProperty {
	return propertyattributedto.NewActivityStreamsAttributedToProperty()
}

// NewActivityStreamsActivityStreamsAudienceProperty creates a new
// ActivityStreamsAudienceProperty
func NewActivityStreamsAudienceProperty() vocab.ActivityStreamsAudienceProperty {
	return propertyaudience.NewActivityStreamsAudienceProperty()
}

// NewActivityStreamsActivityStreamsBccProperty creates a new
// ActivityStreamsBccProperty
func NewActivityStreamsBccProperty() vocab.ActivityStreamsBccProperty {
	return propertybcc.NewActivityStreamsBccProperty()
}

// NewActivityStreamsActivityStreamsBtoProperty creates a new
// ActivityStreamsBtoProperty
func NewActivityStreamsBtoProperty() vocab.ActivityStreamsBtoProperty {
	return propertybto.NewActivityStreamsBtoProperty()
}

// NewActivityStreamsActivityStreamsCcProperty creates a new
// ActivityStreamsCcProperty
func NewActivityStreamsCcProperty() vocab.ActivityStreamsCcProperty {
	return propertycc.NewActivityStreamsCcProperty()
}

// NewActivityStreamsActivityStreamsClosedProperty creates a new
// ActivityStreamsClosedProperty
func NewActivityStreamsClosedProperty() vocab.ActivityStreamsClosedProperty {
	return propertyclosed.NewActivityStreamsClosedProperty()
}

// NewActivityStreamsActivityStreamsContentProperty creates a new
// ActivityStreamsContentProperty
func NewActivityStreamsContentProperty() vocab.ActivityStreamsContentProperty {
	return propertycontent.NewActivityStreamsContentProperty()
}

// NewActivityStreamsActivityStreamsContextProperty creates a new
// ActivityStreamsContextProperty
func NewActivityStreamsContextProperty() vocab.ActivityStreamsContextProperty {
	return propertycontext.NewActivityStreamsContextProperty()
}

// NewActivityStreamsActivityStreamsCurrentProperty creates a new
// ActivityStreamsCurrentProperty
func NewActivityStreamsCurrentProperty() vocab.ActivityStreamsCurrentProperty {
	return propertycurrent.NewActivityStreamsCurrentProperty()
}

// NewActivityStreamsActivityStreamsDeletedProperty creates a new
// ActivityStreamsDeletedProperty
func NewActivityStreamsDeletedProperty() vocab.ActivityStreamsDeletedProperty {
	return propertydeleted.NewActivityStreamsDeletedProperty()
}

// NewActivityStreamsActivityStreamsDescribesProperty creates a new
// ActivityStreamsDescribesProperty
func NewActivityStreamsDescribesProperty() vocab.ActivityStreamsDescribesProperty {
	return propertydescribes.NewActivityStreamsDescribesProperty()
}

// NewActivityStreamsActivityStreamsDurationProperty creates a new
// ActivityStreamsDurationProperty
func NewActivityStreamsDurationProperty() vocab.ActivityStreamsDurationProperty {
	return propertyduration.NewActivityStreamsDurationProperty()
}

// NewActivityStreamsActivityStreamsEndTimeProperty creates a new
// ActivityStreamsEndTimeProperty
func NewActivityStreamsEndTimeProperty() vocab.ActivityStreamsEndTimeProperty {
	return propertyendtime.NewActivityStreamsEndTimeProperty()
}

// NewActivityStreamsActivityStreamsEndpointsProperty creates a new
// ActivityStreamsEndpointsProperty
func NewActivityStreamsEndpointsProperty() vocab.ActivityStreamsEndpointsProperty {
	return propertyendpoints.NewActivityStreamsEndpointsProperty()
}

// NewActivityStreamsActivityStreamsFirstProperty creates a new
// ActivityStreamsFirstProperty
func NewActivityStreamsFirstProperty() vocab.ActivityStreamsFirstProperty {
	return propertyfirst.NewActivityStreamsFirstProperty()
}

// NewActivityStreamsActivityStreamsFollowersProperty creates a new
// ActivityStreamsFollowersProperty
func NewActivityStreamsFollowersProperty() vocab.ActivityStreamsFollowersProperty {
	return propertyfollowers.NewActivityStreamsFollowersProperty()
}

// NewActivityStreamsActivityStreamsFollowingProperty creates a new
// ActivityStreamsFollowingProperty
func NewActivityStreamsFollowingProperty() vocab.ActivityStreamsFollowingProperty {
	return propertyfollowing.NewActivityStreamsFollowingProperty()
}

// NewActivityStreamsActivityStreamsFormerTypeProperty creates a new
// ActivityStreamsFormerTypeProperty
func NewActivityStreamsFormerTypeProperty() vocab.ActivityStreamsFormerTypeProperty {
	return propertyformertype.NewActivityStreamsFormerTypeProperty()
}

// NewActivityStreamsActivityStreamsGeneratorProperty creates a new
// ActivityStreamsGeneratorProperty
func NewActivityStreamsGeneratorProperty() vocab.ActivityStreamsGeneratorProperty {
	return propertygenerator.NewActivityStreamsGeneratorProperty()
}

// NewActivityStreamsActivityStreamsHeightProperty creates a new
// ActivityStreamsHeightProperty
func NewActivityStreamsHeightProperty() vocab.ActivityStreamsHeightProperty {
	return propertyheight.NewActivityStreamsHeightProperty()
}

// NewActivityStreamsActivityStreamsHrefProperty creates a new
// ActivityStreamsHrefProperty
func NewActivityStreamsHrefProperty() vocab.ActivityStreamsHrefProperty {
	return propertyhref.NewActivityStreamsHrefProperty()
}

// NewActivityStreamsActivityStreamsHreflangProperty creates a new
// ActivityStreamsHreflangProperty
func NewActivityStreamsHreflangProperty() vocab.ActivityStreamsHreflangProperty {
	return propertyhreflang.NewActivityStreamsHreflangProperty()
}

// NewActivityStreamsActivityStreamsIconProperty creates a new
// ActivityStreamsIconProperty
func NewActivityStreamsIconProperty() vocab.ActivityStreamsIconProperty {
	return propertyicon.NewActivityStreamsIconProperty()
}

// NewActivityStreamsActivityStreamsImageProperty creates a new
// ActivityStreamsImageProperty
func NewActivityStreamsImageProperty() vocab.ActivityStreamsImageProperty {
	return propertyimage.NewActivityStreamsImageProperty()
}

// NewActivityStreamsActivityStreamsInReplyToProperty creates a new
// ActivityStreamsInReplyToProperty
func NewActivityStreamsInReplyToProperty() vocab.ActivityStreamsInReplyToProperty {
	return propertyinreplyto.NewActivityStreamsInReplyToProperty()
}

// NewActivityStreamsActivityStreamsInboxProperty creates a new
// ActivityStreamsInboxProperty
func NewActivityStreamsInboxProperty() vocab.ActivityStreamsInboxProperty {
	return propertyinbox.NewActivityStreamsInboxProperty()
}

// NewActivityStreamsActivityStreamsInstrumentProperty creates a new
// ActivityStreamsInstrumentProperty
func NewActivityStreamsInstrumentProperty() vocab.ActivityStreamsInstrumentProperty {
	return propertyinstrument.NewActivityStreamsInstrumentProperty()
}

// NewActivityStreamsActivityStreamsItemsProperty creates a new
// ActivityStreamsItemsProperty
func NewActivityStreamsItemsProperty() vocab.ActivityStreamsItemsProperty {
	return propertyitems.NewActivityStreamsItemsProperty()
}

// NewActivityStreamsActivityStreamsLastProperty creates a new
// ActivityStreamsLastProperty
func NewActivityStreamsLastProperty() vocab.ActivityStreamsLastProperty {
	return propertylast.NewActivityStreamsLastProperty()
}

// NewActivityStreamsActivityStreamsLatitudeProperty creates a new
// ActivityStreamsLatitudeProperty
func NewActivityStreamsLatitudeProperty() vocab.ActivityStreamsLatitudeProperty {
	return propertylatitude.NewActivityStreamsLatitudeProperty()
}

// NewActivityStreamsActivityStreamsLikedProperty creates a new
// ActivityStreamsLikedProperty
func NewActivityStreamsLikedProperty() vocab.ActivityStreamsLikedProperty {
	return propertyliked.NewActivityStreamsLikedProperty()
}

// NewActivityStreamsActivityStreamsLikesProperty creates a new
// ActivityStreamsLikesProperty
func NewActivityStreamsLikesProperty() vocab.ActivityStreamsLikesProperty {
	return propertylikes.NewActivityStreamsLikesProperty()
}

// NewActivityStreamsActivityStreamsLocationProperty creates a new
// ActivityStreamsLocationProperty
func NewActivityStreamsLocationProperty() vocab.ActivityStreamsLocationProperty {
	return propertylocation.NewActivityStreamsLocationProperty()
}

// NewActivityStreamsActivityStreamsLongitudeProperty creates a new
// ActivityStreamsLongitudeProperty
func NewActivityStreamsLongitudeProperty() vocab.ActivityStreamsLongitudeProperty {
	return propertylongitude.NewActivityStreamsLongitudeProperty()
}

// NewActivityStreamsActivityStreamsManuallyApprovesFollowersProperty creates a
// new ActivityStreamsManuallyApprovesFollowersProperty
func NewActivityStreamsManuallyApprovesFollowersProperty() vocab.ActivityStreamsManuallyApprovesFollowersProperty {
	return propertymanuallyapprovesfollowers.NewActivityStreamsManuallyApprovesFollowersProperty()
}

// NewActivityStreamsActivityStreamsMediaTypeProperty creates a new
// ActivityStreamsMediaTypeProperty
func NewActivityStreamsMediaTypeProperty() vocab.ActivityStreamsMediaTypeProperty {
	return propertymediatype.NewActivityStreamsMediaTypeProperty()
}

// NewActivityStreamsActivityStreamsMovedToProperty creates a new
// ActivityStreamsMovedToProperty
func NewActivityStreamsMovedToProperty() vocab.ActivityStreamsMovedToProperty {
	return propertymovedto.NewActivityStreamsMovedToProperty()
}

// NewActivityStreamsActivityStreamsNameProperty creates a new
// ActivityStreamsNameProperty
func NewActivityStreamsNameProperty() vocab.ActivityStreamsNameProperty {
	return propertyname.NewActivityStreamsNameProperty()
}

// NewActivityStreamsActivityStreamsNextProperty creates a new
// ActivityStreamsNextProperty
func NewActivityStreamsNextProperty() vocab.ActivityStreamsNextProperty {
	return propertynext.NewActivityStreamsNextProperty()
}

// NewActivityStreamsActivityStreamsObjectProperty creates a new
// ActivityStreamsObjectProperty
func NewActivityStreamsObjectProperty() vocab.ActivityStreamsObjectProperty {
	return propertyobject.NewActivityStreamsObjectProperty()
}

// NewActivityStreamsActivityStreamsOneOfProperty creates a new
// ActivityStreamsOneOfProperty
func NewActivityStreamsOneOfProperty() vocab.ActivityStreamsOneOfProperty {
	return propertyoneof.NewActivityStreamsOneOfProperty()
}

// NewActivityStreamsActivityStreamsOrderedItemsProperty creates a new
// ActivityStreamsOrderedItemsProperty
func NewActivityStreamsOrderedItemsProperty() vocab.ActivityStreamsOrderedItemsProperty {
	return propertyordereditems.NewActivityStreamsOrderedItemsProperty()
}

// NewActivityStreamsActivityStreamsOriginProperty creates a new
// ActivityStreamsOriginProperty
func NewActivityStreamsOriginProperty() vocab.ActivityStreamsOriginProperty {
	return propertyorigin.NewActivityStreamsOriginProperty()
}

// NewActivityStreamsActivityStreamsOutboxProperty creates a new
// ActivityStreamsOutboxProperty
func NewActivityStreamsOutboxProperty() vocab.ActivityStreamsOutboxProperty {
	return propertyoutbox.NewActivityStreamsOutboxProperty()
}

// NewActivityStreamsActivityStreamsPartOfProperty creates a new
// ActivityStreamsPartOfProperty
func NewActivityStreamsPartOfProperty() vocab.ActivityStreamsPartOfProperty {
	return propertypartof.NewActivityStreamsPartOfProperty()
}

// NewActivityStreamsActivityStreamsPreferredUsernameProperty creates a new
// ActivityStreamsPreferredUsernameProperty
func NewActivityStreamsPreferredUsernameProperty() vocab.ActivityStreamsPreferredUsernameProperty {
	return propertypreferredusername.NewActivityStreamsPreferredUsernameProperty()
}

// NewActivityStreamsActivityStreamsPrevProperty creates a new
// ActivityStreamsPrevProperty
func NewActivityStreamsPrevProperty() vocab.ActivityStreamsPrevProperty {
	return propertyprev.NewActivityStreamsPrevProperty()
}

// NewActivityStreamsActivityStreamsPreviewProperty creates a new
// ActivityStreamsPreviewProperty
func NewActivityStreamsPreviewProperty() vocab.ActivityStreamsPreviewProperty {
	return propertypreview.NewActivityStreamsPreviewProperty()
}

// NewActivityStreamsActivityStreamsPublishedProperty creates a new
// ActivityStreamsPublishedProperty
func NewActivityStreamsPublishedProperty() vocab.ActivityStreamsPublishedProperty {
	return propertypublished.NewActivityStreamsPublishedProperty()
}

// NewActivityStreamsActivityStreamsRadiusProperty creates a new
// ActivityStreamsRadiusProperty
func NewActivityStreamsRadiusProperty() vocab.ActivityStreamsRadiusProperty {
	return propertyradius.NewActivityStreamsRadiusProperty()
}

// NewActivityStreamsActivityStreamsRelProperty creates a new
// ActivityStreamsRelProperty
func NewActivityStreamsRelProperty() vocab.ActivityStreamsRelProperty {
	return propertyrel.NewActivityStreamsRelProperty()
}

// NewActivityStreamsActivityStreamsRelationshipProperty creates a new
// ActivityStreamsRelationshipProperty
func NewActivityStreamsRelationshipProperty() vocab.ActivityStreamsRelationshipProperty {
	return propertyrelationship.NewActivityStreamsRelationshipProperty()
}

// NewActivityStreamsActivityStreamsRepliesProperty creates a new
// ActivityStreamsRepliesProperty
func NewActivityStreamsRepliesProperty() vocab.ActivityStreamsRepliesProperty {
	return propertyreplies.NewActivityStreamsRepliesProperty()
}

// NewActivityStreamsActivityStreamsResultProperty creates a new
// ActivityStreamsResultProperty
func NewActivityStreamsResultProperty() vocab.ActivityStreamsResultProperty {
	return propertyresult.NewActivityStreamsResultProperty()
}

// NewActivityStreamsActivityStreamsSensitiveProperty creates a new
// ActivityStreamsSensitiveProperty
func NewActivityStreamsSensitiveProperty() vocab.ActivityStreamsSensitiveProperty {
	return propertysensitive.NewActivityStreamsSensitiveProperty()
}

// NewActivityStreamsActivityStreamsSharedInboxProperty creates a new
// ActivityStreamsSharedInboxProperty
func NewActivityStreamsSharedInboxProperty() vocab.ActivityStreamsSharedInboxProperty {
	return propertysharedinbox.NewActivityStreamsSharedInboxProperty()
}

// NewActivityStreamsActivityStreamsSharesProperty creates a new
// ActivityStreamsSharesProperty
func NewActivityStreamsSharesProperty() vocab.ActivityStreamsSharesProperty {
	return propertyshares.NewActivityStreamsSharesProperty()
}

// NewActivityStreamsActivityStreamsSourceProperty creates a new
// ActivityStreamsSourceProperty
func NewActivityStreamsSourceProperty() vocab.ActivityStreamsSourceProperty {
	return propertysource.NewActivityStreamsSourceProperty()
}

// NewActivityStreamsActivityStreamsStartIndexProperty creates a new
// ActivityStreamsStartIndexProperty
func NewActivityStreamsStartIndexProperty() vocab.ActivityStreamsStartIndexProperty {
	return propertystartindex.NewActivityStreamsStartIndexProperty()
}

// NewActivityStreamsActivityStreamsStartTimeProperty creates a new
// ActivityStreamsStartTimeProperty
func NewActivityStreamsStartTimeProperty() vocab.ActivityStreamsStartTimeProperty {
	return propertystarttime.NewActivityStreamsStartTimeProperty()
}

// NewActivityStreamsActivityStreamsStreamsProperty creates a new
// ActivityStreamsStreamsProperty
func NewActivityStreamsStreamsProperty() vocab.ActivityStreamsStreamsProperty {
	return propertystreams.NewActivityStreamsStreamsProperty()
}

// NewActivityStreamsActivityStreamsSubjectProperty creates a new
// ActivityStreamsSubjectProperty
func NewActivityStreamsSubjectProperty() vocab.ActivityStreamsSubjectProperty {
	return propertysubject.NewActivityStreamsSubjectProperty()
}

// NewActivityStreamsActivityStreamsSummaryProperty creates a new
// ActivityStreamsSummaryProperty
func NewActivityStreamsSummaryProperty() vocab.ActivityStreamsSummaryProperty {
	return propertysummary.NewActivityStreamsSummaryProperty()
}

// NewActivityStreamsActivityStreamsTagProperty creates a new
// ActivityStreamsTagProperty
func NewActivityStreamsTagProperty() vocab.ActivityStreamsTagProperty {
	return propertytag.NewActivityStreamsTagProperty()
}

// NewActivityStreamsActivityStreamsTargetProperty creates a new
// ActivityStreamsTargetProperty
func NewActivityStreamsTargetProperty() vocab.ActivityStreamsTargetProperty {
	return propertytarget.NewActivityStreamsTargetProperty()
}

// NewActivityStreamsActivityStreamsToProperty creates a new
// ActivityStreamsToProperty
func NewActivityStreamsToProperty() vocab.ActivityStreamsToProperty {
	return propertyto.NewActivityStreamsToProperty()
}

// NewActivityStreamsActivityStreamsTotalItemsProperty creates a new
// ActivityStreamsTotalItemsProperty
func NewActivityStreamsTotalItemsProperty() vocab.ActivityStreamsTotalItemsProperty {
	return propertytotalitems.NewActivityStreamsTotalItemsProperty()
}

// NewActivityStreamsActivityStreamsUnitsProperty creates a new
// ActivityStreamsUnitsProperty
func NewActivityStreamsUnitsProperty() vocab.ActivityStreamsUnitsProperty {
	return propertyunits.NewActivityStreamsUnitsProperty()
}

// NewActivityStreamsActivityStreamsUpdatedProperty creates a new
// ActivityStreamsUpdatedProperty
func NewActivityStreamsUpdatedProperty() vocab.ActivityStreamsUpdatedProperty {
	return propertyupdated.NewActivityStreamsUpdatedProperty()
}

// NewActivityStreamsActivityStreamsUrlProperty creates a new
// ActivityStreamsUrlProperty
func NewActivityStreamsUrlProperty() vocab.ActivityStreamsUrlProperty {
	return propertyurl.NewActivityStreamsUrlProperty()
}

// NewActivityStreamsActivityStreamsWidthProperty creates a new
// ActivityStreamsWidthProperty
func NewActivityStreamsWidthProperty() vocab.ActivityStreamsWidthProperty {
	return propertywidth.NewActivityStreamsWidthProperty()
}
