// Code generated by astool. DO NOT EDIT.

package streams

import (
	typeemojireact "code.superseriousbusiness.org/activity/streams/impl/litepub/type_emojireact"
	vocab "code.superseriousbusiness.org/activity/streams/vocab"
)

// IsOrExtendsLitePubEmojiReact returns true if the other provided type is the
// EmojiReact type or extends from the EmojiReact type.
func IsOrExtendsLitePubEmojiReact(other vocab.Type) bool {
	return typeemojireact.IsOrExtendsEmojiReact(other)
}
