// Code generated by astool. DO NOT EDIT.

package streams

import (
	propertyvalue "code.superseriousbusiness.org/activity/streams/impl/schema/property_value"
	vocab "code.superseriousbusiness.org/activity/streams/vocab"
)

// NewSchemaSchemaValueProperty creates a new SchemaValueProperty
func NewSchemaValueProperty() vocab.SchemaValueProperty {
	return propertyvalue.NewSchemaValueProperty()
}
