// Code generated by astool. DO NOT EDIT.

package vocab

import "net/url"

// ActivityStreamsSensitivePropertyIterator represents a single value for the
// "sensitive" property.
type ActivityStreamsSensitivePropertyIterator interface {
	// Get returns the value of this property. When IsXMLSchemaBoolean returns
	// false, Get will return any arbitrary value.
	Get() bool
	// GetIRI returns the IRI of this property. When IsIRI returns false,
	// GetIRI will return any arbitrary value.
	GetIRI() *url.URL
	// HasAny returns true if the value or IRI is set.
	HasAny() bool
	// IsIRI returns true if this property is an IRI.
	IsIRI() bool
	// IsXMLSchemaBoolean returns true if this property is set and not an IRI.
	IsXMLSchemaBoolean() bool
	// JSONLDContext returns the JSONLD URIs required in the context string
	// for this property and the specific values that are set. The value
	// in the map is the alias used to import the property's value or
	// values.
	JSONLDContext() map[string]string
	// KindIndex computes an arbitrary value for indexing this kind of value.
	// This is a leaky API detail only for folks looking to replace the
	// go-fed implementation. Applications should not use this method.
	KindIndex() int
	// LessThan compares two instances of this property with an arbitrary but
	// stable comparison. Applications should not use this because it is
	// only meant to help alternative implementations to go-fed to be able
	// to normalize nonfunctional properties.
	LessThan(o ActivityStreamsSensitivePropertyIterator) bool
	// Name returns the name of this property: "ActivityStreamsSensitive".
	Name() string
	// Next returns the next iterator, or nil if there is no next iterator.
	Next() ActivityStreamsSensitivePropertyIterator
	// Prev returns the previous iterator, or nil if there is no previous
	// iterator.
	Prev() ActivityStreamsSensitivePropertyIterator
	// Set sets the value of this property. Calling IsXMLSchemaBoolean
	// afterwards will return true.
	Set(v bool)
	// SetIRI sets the value of this property. Calling IsIRI afterwards will
	// return true.
	SetIRI(v *url.URL)
}

// Indicates that some users may wish to apply discretion about viewing this
// object's content, whether due to nudity, violence, or any other likely
// aspects that viewers may be sensitive to.
type ActivityStreamsSensitiveProperty interface {
	// AppendIRI appends an IRI value to the back of a list of the property
	// "sensitive"
	AppendIRI(v *url.URL)
	// AppendXMLSchemaBoolean appends a boolean value to the back of a list of
	// the property "sensitive". Invalidates iterators that are traversing
	// using Prev.
	AppendXMLSchemaBoolean(v bool)
	// At returns the property value for the specified index. Panics if the
	// index is out of bounds.
	At(index int) ActivityStreamsSensitivePropertyIterator
	// Begin returns the first iterator, or nil if empty. Can be used with the
	// iterator's Next method and this property's End method to iterate
	// from front to back through all values.
	Begin() ActivityStreamsSensitivePropertyIterator
	// Empty returns returns true if there are no elements.
	Empty() bool
	// End returns beyond-the-last iterator, which is nil. Can be used with
	// the iterator's Next method and this property's Begin method to
	// iterate from front to back through all values.
	End() ActivityStreamsSensitivePropertyIterator
	// Insert inserts an IRI value at the specified index for a property
	// "sensitive". Existing elements at that index and higher are shifted
	// back once. Invalidates all iterators.
	InsertIRI(idx int, v *url.URL)
	// InsertXMLSchemaBoolean inserts a boolean value at the specified index
	// for a property "sensitive". Existing elements at that index and
	// higher are shifted back once. Invalidates all iterators.
	InsertXMLSchemaBoolean(idx int, v bool)
	// JSONLDContext returns the JSONLD URIs required in the context string
	// for this property and the specific values that are set. The value
	// in the map is the alias used to import the property's value or
	// values.
	JSONLDContext() map[string]string
	// KindIndex computes an arbitrary value for indexing this kind of value.
	// This is a leaky API method specifically needed only for alternate
	// implementations for go-fed. Applications should not use this
	// method. Panics if the index is out of bounds.
	KindIndex(idx int) int
	// Len returns the number of values that exist for the "sensitive"
	// property.
	Len() (length int)
	// Less computes whether another property is less than this one. Mixing
	// types results in a consistent but arbitrary ordering
	Less(i, j int) bool
	// LessThan compares two instances of this property with an arbitrary but
	// stable comparison. Applications should not use this because it is
	// only meant to help alternative implementations to go-fed to be able
	// to normalize nonfunctional properties.
	LessThan(o ActivityStreamsSensitiveProperty) bool
	// Name returns the name of this property ("sensitive") with any alias.
	Name() string
	// PrependIRI prepends an IRI value to the front of a list of the property
	// "sensitive".
	PrependIRI(v *url.URL)
	// PrependXMLSchemaBoolean prepends a boolean value to the front of a list
	// of the property "sensitive". Invalidates all iterators.
	PrependXMLSchemaBoolean(v bool)
	// Remove deletes an element at the specified index from a list of the
	// property "sensitive", regardless of its type. Panics if the index
	// is out of bounds. Invalidates all iterators.
	Remove(idx int)
	// Serialize converts this into an interface representation suitable for
	// marshalling into a text or binary format. Applications should not
	// need this function as most typical use cases serialize types
	// instead of individual properties. It is exposed for alternatives to
	// go-fed implementations to use.
	Serialize() (interface{}, error)
	// Set sets a boolean value to be at the specified index for the property
	// "sensitive". Panics if the index is out of bounds. Invalidates all
	// iterators.
	Set(idx int, v bool)
	// SetIRI sets an IRI value to be at the specified index for the property
	// "sensitive". Panics if the index is out of bounds.
	SetIRI(idx int, v *url.URL)
	// Swap swaps the location of values at two indices for the "sensitive"
	// property.
	Swap(i, j int)
}
