// Code generated by astool. DO NOT EDIT.

package streams

import (
	typealbum "code.superseriousbusiness.org/activity/streams/impl/funkwhale/type_album"
	typeartist "code.superseriousbusiness.org/activity/streams/impl/funkwhale/type_artist"
	typelibrary "code.superseriousbusiness.org/activity/streams/impl/funkwhale/type_library"
	typetrack "code.superseriousbusiness.org/activity/streams/impl/funkwhale/type_track"
	vocab "code.superseriousbusiness.org/activity/streams/vocab"
)

// NewFunkwhaleAlbum creates a new FunkwhaleAlbum
func NewFunkwhaleAlbum() vocab.FunkwhaleAlbum {
	return typealbum.NewFunkwhaleAlbum()
}

// NewFunkwhaleArtist creates a new FunkwhaleArtist
func NewFunkwhaleArtist() vocab.FunkwhaleArtist {
	return typeartist.NewFunkwhaleArtist()
}

// NewFunkwhaleLibrary creates a new FunkwhaleLibrary
func NewFunkwhaleLibrary() vocab.FunkwhaleLibrary {
	return typelibrary.NewFunkwhaleLibrary()
}

// NewFunkwhaleTrack creates a new FunkwhaleTrack
func NewFunkwhaleTrack() vocab.FunkwhaleTrack {
	return typetrack.NewFunkwhaleTrack()
}
