// Code generated by astool. DO NOT EDIT.

package streams

import (
	typeemojireact "code.superseriousbusiness.org/activity/streams/impl/litepub/type_emojireact"
	vocab "code.superseriousbusiness.org/activity/streams/vocab"
)

// LitePubLitePubEmojiReactExtends returns true if EmojiReact extends from the
// other's type.
func LitePubLitePubEmojiReactExtends(other vocab.Type) bool {
	return typeemojireact.LitePubEmojiReactExtends(other)
}
