// Code generated by astool. DO NOT EDIT.

package streams

import (
	typeannounceapproval "code.superseriousbusiness.org/activity/streams/impl/gotosocial/type_announceapproval"
	typeannounceauthorization "code.superseriousbusiness.org/activity/streams/impl/gotosocial/type_announceauthorization"
	typeannouncerequest "code.superseriousbusiness.org/activity/streams/impl/gotosocial/type_announcerequest"
	typecanannounce "code.superseriousbusiness.org/activity/streams/impl/gotosocial/type_canannounce"
	typecanlike "code.superseriousbusiness.org/activity/streams/impl/gotosocial/type_canlike"
	typecanquote "code.superseriousbusiness.org/activity/streams/impl/gotosocial/type_canquote"
	typecanreply "code.superseriousbusiness.org/activity/streams/impl/gotosocial/type_canreply"
	typeinteractionpolicy "code.superseriousbusiness.org/activity/streams/impl/gotosocial/type_interactionpolicy"
	typelikeapproval "code.superseriousbusiness.org/activity/streams/impl/gotosocial/type_likeapproval"
	typelikeauthorization "code.superseriousbusiness.org/activity/streams/impl/gotosocial/type_likeauthorization"
	typelikerequest "code.superseriousbusiness.org/activity/streams/impl/gotosocial/type_likerequest"
	typequoteauthorization "code.superseriousbusiness.org/activity/streams/impl/gotosocial/type_quoteauthorization"
	typequoterequest "code.superseriousbusiness.org/activity/streams/impl/gotosocial/type_quoterequest"
	typereplyapproval "code.superseriousbusiness.org/activity/streams/impl/gotosocial/type_replyapproval"
	typereplyauthorization "code.superseriousbusiness.org/activity/streams/impl/gotosocial/type_replyauthorization"
	typereplyrequest "code.superseriousbusiness.org/activity/streams/impl/gotosocial/type_replyrequest"
	vocab "code.superseriousbusiness.org/activity/streams/vocab"
)

// GoToSocialGoToSocialAnnounceApprovalExtends returns true if AnnounceApproval
// extends from the other's type.
func GoToSocialGoToSocialAnnounceApprovalExtends(other vocab.Type) bool {
	return typeannounceapproval.GoToSocialAnnounceApprovalExtends(other)
}

// GoToSocialGoToSocialAnnounceAuthorizationExtends returns true if
// AnnounceAuthorization extends from the other's type.
func GoToSocialGoToSocialAnnounceAuthorizationExtends(other vocab.Type) bool {
	return typeannounceauthorization.GoToSocialAnnounceAuthorizationExtends(other)
}

// GoToSocialGoToSocialAnnounceRequestExtends returns true if AnnounceRequest
// extends from the other's type.
func GoToSocialGoToSocialAnnounceRequestExtends(other vocab.Type) bool {
	return typeannouncerequest.GoToSocialAnnounceRequestExtends(other)
}

// GoToSocialGoToSocialCanAnnounceExtends returns true if CanAnnounce extends from
// the other's type.
func GoToSocialGoToSocialCanAnnounceExtends(other vocab.Type) bool {
	return typecanannounce.GoToSocialCanAnnounceExtends(other)
}

// GoToSocialGoToSocialCanLikeExtends returns true if CanLike extends from the
// other's type.
func GoToSocialGoToSocialCanLikeExtends(other vocab.Type) bool {
	return typecanlike.GoToSocialCanLikeExtends(other)
}

// GoToSocialGoToSocialCanQuoteExtends returns true if CanQuote extends from the
// other's type.
func GoToSocialGoToSocialCanQuoteExtends(other vocab.Type) bool {
	return typecanquote.GoToSocialCanQuoteExtends(other)
}

// GoToSocialGoToSocialCanReplyExtends returns true if CanReply extends from the
// other's type.
func GoToSocialGoToSocialCanReplyExtends(other vocab.Type) bool {
	return typecanreply.GoToSocialCanReplyExtends(other)
}

// GoToSocialGoToSocialInteractionPolicyExtends returns true if InteractionPolicy
// extends from the other's type.
func GoToSocialGoToSocialInteractionPolicyExtends(other vocab.Type) bool {
	return typeinteractionpolicy.GoToSocialInteractionPolicyExtends(other)
}

// GoToSocialGoToSocialLikeApprovalExtends returns true if LikeApproval extends
// from the other's type.
func GoToSocialGoToSocialLikeApprovalExtends(other vocab.Type) bool {
	return typelikeapproval.GoToSocialLikeApprovalExtends(other)
}

// GoToSocialGoToSocialLikeAuthorizationExtends returns true if LikeAuthorization
// extends from the other's type.
func GoToSocialGoToSocialLikeAuthorizationExtends(other vocab.Type) bool {
	return typelikeauthorization.GoToSocialLikeAuthorizationExtends(other)
}

// GoToSocialGoToSocialLikeRequestExtends returns true if LikeRequest extends from
// the other's type.
func GoToSocialGoToSocialLikeRequestExtends(other vocab.Type) bool {
	return typelikerequest.GoToSocialLikeRequestExtends(other)
}

// GoToSocialGoToSocialQuoteAuthorizationExtends returns true if
// QuoteAuthorization extends from the other's type.
func GoToSocialGoToSocialQuoteAuthorizationExtends(other vocab.Type) bool {
	return typequoteauthorization.GoToSocialQuoteAuthorizationExtends(other)
}

// GoToSocialGoToSocialQuoteRequestExtends returns true if QuoteRequest extends
// from the other's type.
func GoToSocialGoToSocialQuoteRequestExtends(other vocab.Type) bool {
	return typequoterequest.GoToSocialQuoteRequestExtends(other)
}

// GoToSocialGoToSocialReplyApprovalExtends returns true if ReplyApproval extends
// from the other's type.
func GoToSocialGoToSocialReplyApprovalExtends(other vocab.Type) bool {
	return typereplyapproval.GoToSocialReplyApprovalExtends(other)
}

// GoToSocialGoToSocialReplyAuthorizationExtends returns true if
// ReplyAuthorization extends from the other's type.
func GoToSocialGoToSocialReplyAuthorizationExtends(other vocab.Type) bool {
	return typereplyauthorization.GoToSocialReplyAuthorizationExtends(other)
}

// GoToSocialGoToSocialReplyRequestExtends returns true if ReplyRequest extends
// from the other's type.
func GoToSocialGoToSocialReplyRequestExtends(other vocab.Type) bool {
	return typereplyrequest.GoToSocialReplyRequestExtends(other)
}
