// Code generated by astool. DO NOT EDIT.

package streams

import (
	typeemojireact "code.superseriousbusiness.org/activity/streams/impl/litepub/type_emojireact"
	vocab "code.superseriousbusiness.org/activity/streams/vocab"
)

// LitePubEmojiReactIsExtendedBy returns true if the other's type extends from
// EmojiReact. Note that it returns false if the types are the same; see the
// "IsOrExtends" variant instead.
func LitePubEmojiReactIsExtendedBy(other vocab.Type) bool {
	return typeemojireact.EmojiReactIsExtendedBy(other)
}
