// Code generated by astool. DO NOT EDIT.

package streams

import (
	propertyid "code.superseriousbusiness.org/activity/streams/impl/jsonld/property_id"
	propertytype "code.superseriousbusiness.org/activity/streams/impl/jsonld/property_type"
	vocab "code.superseriousbusiness.org/activity/streams/vocab"
)

// NewJSONLDJSONLDTypeProperty creates a new JSONLDTypeProperty
func NewJSONLDTypeProperty() vocab.JSONLDTypeProperty {
	return propertytype.NewJSONLDTypeProperty()
}

// NewJSONLDJSONLDIdProperty creates a new JSONLDIdProperty
func NewJSONLDIdProperty() vocab.JSONLDIdProperty {
	return propertyid.NewJSONLDIdProperty()
}
