// Code generated by astool. DO NOT EDIT.

package streams

import (
	propertyalways "code.superseriousbusiness.org/activity/streams/impl/gotosocial/property_always"
	propertyannounceauthorization "code.superseriousbusiness.org/activity/streams/impl/gotosocial/property_announceauthorization"
	propertyapprovalrequired "code.superseriousbusiness.org/activity/streams/impl/gotosocial/property_approvalrequired"
	propertyapprovedby "code.superseriousbusiness.org/activity/streams/impl/gotosocial/property_approvedby"
	propertyautomaticapproval "code.superseriousbusiness.org/activity/streams/impl/gotosocial/property_automaticapproval"
	propertycanannounce "code.superseriousbusiness.org/activity/streams/impl/gotosocial/property_canannounce"
	propertycanlike "code.superseriousbusiness.org/activity/streams/impl/gotosocial/property_canlike"
	propertycanquote "code.superseriousbusiness.org/activity/streams/impl/gotosocial/property_canquote"
	propertycanreply "code.superseriousbusiness.org/activity/streams/impl/gotosocial/property_canreply"
	propertyhidesccpublicfromunauthedweb "code.superseriousbusiness.org/activity/streams/impl/gotosocial/property_hidesccpublicfromunauthedweb"
	propertyhidestopublicfromunauthedweb "code.superseriousbusiness.org/activity/streams/impl/gotosocial/property_hidestopublicfromunauthedweb"
	propertyinteractingobject "code.superseriousbusiness.org/activity/streams/impl/gotosocial/property_interactingobject"
	propertyinteractionpolicy "code.superseriousbusiness.org/activity/streams/impl/gotosocial/property_interactionpolicy"
	propertyinteractiontarget "code.superseriousbusiness.org/activity/streams/impl/gotosocial/property_interactiontarget"
	propertylikeauthorization "code.superseriousbusiness.org/activity/streams/impl/gotosocial/property_likeauthorization"
	propertymanualapproval "code.superseriousbusiness.org/activity/streams/impl/gotosocial/property_manualapproval"
	propertyquoteauthorization "code.superseriousbusiness.org/activity/streams/impl/gotosocial/property_quoteauthorization"
	propertyreplyauthorization "code.superseriousbusiness.org/activity/streams/impl/gotosocial/property_replyauthorization"
	vocab "code.superseriousbusiness.org/activity/streams/vocab"
)

// NewGoToSocialGoToSocialAlwaysProperty creates a new GoToSocialAlwaysProperty
func NewGoToSocialAlwaysProperty() vocab.GoToSocialAlwaysProperty {
	return propertyalways.NewGoToSocialAlwaysProperty()
}

// NewGoToSocialGoToSocialAnnounceAuthorizationProperty creates a new
// GoToSocialAnnounceAuthorizationProperty
func NewGoToSocialAnnounceAuthorizationProperty() vocab.GoToSocialAnnounceAuthorizationProperty {
	return propertyannounceauthorization.NewGoToSocialAnnounceAuthorizationProperty()
}

// NewGoToSocialGoToSocialApprovalRequiredProperty creates a new
// GoToSocialApprovalRequiredProperty
func NewGoToSocialApprovalRequiredProperty() vocab.GoToSocialApprovalRequiredProperty {
	return propertyapprovalrequired.NewGoToSocialApprovalRequiredProperty()
}

// NewGoToSocialGoToSocialApprovedByProperty creates a new
// GoToSocialApprovedByProperty
func NewGoToSocialApprovedByProperty() vocab.GoToSocialApprovedByProperty {
	return propertyapprovedby.NewGoToSocialApprovedByProperty()
}

// NewGoToSocialGoToSocialAutomaticApprovalProperty creates a new
// GoToSocialAutomaticApprovalProperty
func NewGoToSocialAutomaticApprovalProperty() vocab.GoToSocialAutomaticApprovalProperty {
	return propertyautomaticapproval.NewGoToSocialAutomaticApprovalProperty()
}

// NewGoToSocialGoToSocialCanAnnounceProperty creates a new
// GoToSocialCanAnnounceProperty
func NewGoToSocialCanAnnounceProperty() vocab.GoToSocialCanAnnounceProperty {
	return propertycanannounce.NewGoToSocialCanAnnounceProperty()
}

// NewGoToSocialGoToSocialCanLikeProperty creates a new GoToSocialCanLikeProperty
func NewGoToSocialCanLikeProperty() vocab.GoToSocialCanLikeProperty {
	return propertycanlike.NewGoToSocialCanLikeProperty()
}

// NewGoToSocialGoToSocialCanQuoteProperty creates a new GoToSocialCanQuoteProperty
func NewGoToSocialCanQuoteProperty() vocab.GoToSocialCanQuoteProperty {
	return propertycanquote.NewGoToSocialCanQuoteProperty()
}

// NewGoToSocialGoToSocialCanReplyProperty creates a new GoToSocialCanReplyProperty
func NewGoToSocialCanReplyProperty() vocab.GoToSocialCanReplyProperty {
	return propertycanreply.NewGoToSocialCanReplyProperty()
}

// NewGoToSocialGoToSocialHidesCcPublicFromUnauthedWebProperty creates a new
// GoToSocialHidesCcPublicFromUnauthedWebProperty
func NewGoToSocialHidesCcPublicFromUnauthedWebProperty() vocab.GoToSocialHidesCcPublicFromUnauthedWebProperty {
	return propertyhidesccpublicfromunauthedweb.NewGoToSocialHidesCcPublicFromUnauthedWebProperty()
}

// NewGoToSocialGoToSocialHidesToPublicFromUnauthedWebProperty creates a new
// GoToSocialHidesToPublicFromUnauthedWebProperty
func NewGoToSocialHidesToPublicFromUnauthedWebProperty() vocab.GoToSocialHidesToPublicFromUnauthedWebProperty {
	return propertyhidestopublicfromunauthedweb.NewGoToSocialHidesToPublicFromUnauthedWebProperty()
}

// NewGoToSocialGoToSocialInteractingObjectProperty creates a new
// GoToSocialInteractingObjectProperty
func NewGoToSocialInteractingObjectProperty() vocab.GoToSocialInteractingObjectProperty {
	return propertyinteractingobject.NewGoToSocialInteractingObjectProperty()
}

// NewGoToSocialGoToSocialInteractionPolicyProperty creates a new
// GoToSocialInteractionPolicyProperty
func NewGoToSocialInteractionPolicyProperty() vocab.GoToSocialInteractionPolicyProperty {
	return propertyinteractionpolicy.NewGoToSocialInteractionPolicyProperty()
}

// NewGoToSocialGoToSocialInteractionTargetProperty creates a new
// GoToSocialInteractionTargetProperty
func NewGoToSocialInteractionTargetProperty() vocab.GoToSocialInteractionTargetProperty {
	return propertyinteractiontarget.NewGoToSocialInteractionTargetProperty()
}

// NewGoToSocialGoToSocialLikeAuthorizationProperty creates a new
// GoToSocialLikeAuthorizationProperty
func NewGoToSocialLikeAuthorizationProperty() vocab.GoToSocialLikeAuthorizationProperty {
	return propertylikeauthorization.NewGoToSocialLikeAuthorizationProperty()
}

// NewGoToSocialGoToSocialManualApprovalProperty creates a new
// GoToSocialManualApprovalProperty
func NewGoToSocialManualApprovalProperty() vocab.GoToSocialManualApprovalProperty {
	return propertymanualapproval.NewGoToSocialManualApprovalProperty()
}

// NewGoToSocialGoToSocialQuoteAuthorizationProperty creates a new
// GoToSocialQuoteAuthorizationProperty
func NewGoToSocialQuoteAuthorizationProperty() vocab.GoToSocialQuoteAuthorizationProperty {
	return propertyquoteauthorization.NewGoToSocialQuoteAuthorizationProperty()
}

// NewGoToSocialGoToSocialReplyAuthorizationProperty creates a new
// GoToSocialReplyAuthorizationProperty
func NewGoToSocialReplyAuthorizationProperty() vocab.GoToSocialReplyAuthorizationProperty {
	return propertyreplyauthorization.NewGoToSocialReplyAuthorizationProperty()
}
