// Code generated by astool. DO NOT EDIT.

package streams

import (
	typepropertyvalue "code.superseriousbusiness.org/activity/streams/impl/schema/type_propertyvalue"
	vocab "code.superseriousbusiness.org/activity/streams/vocab"
)

// SchemaSchemaPropertyValueExtends returns true if PropertyValue extends from the
// other's type.
func SchemaSchemaPropertyValueExtends(other vocab.Type) bool {
	return typepropertyvalue.SchemaPropertyValueExtends(other)
}
