// Code generated by astool. DO NOT EDIT.

package streams

import (
	typeemojireact "code.superseriousbusiness.org/activity/streams/impl/litepub/type_emojireact"
	vocab "code.superseriousbusiness.org/activity/streams/vocab"
)

// NewLitePubEmojiReact creates a new LitePubEmojiReact
func NewLitePubEmojiReact() vocab.LitePubEmojiReact {
	return typeemojireact.NewLitePubEmojiReact()
}
