// Code generated by astool. DO NOT EDIT.

package streams

import (
	typepropertyvalue "code.superseriousbusiness.org/activity/streams/impl/schema/type_propertyvalue"
	vocab "code.superseriousbusiness.org/activity/streams/vocab"
)

// SchemaPropertyValueIsExtendedBy returns true if the other's type extends from
// PropertyValue. Note that it returns false if the types are the same; see
// the "IsOrExtends" variant instead.
func SchemaPropertyValueIsExtendedBy(other vocab.Type) bool {
	return typepropertyvalue.PropertyValueIsExtendedBy(other)
}
