// Code generated by astool. DO NOT EDIT.

package streams

import (
	typepublickey "code.superseriousbusiness.org/activity/streams/impl/w3idsecurityv1/type_publickey"
	vocab "code.superseriousbusiness.org/activity/streams/vocab"
)

// W3IDSecurityV1W3IDSecurityV1PublicKeyExtends returns true if PublicKey extends
// from the other's type.
func W3IDSecurityV1W3IDSecurityV1PublicKeyExtends(other vocab.Type) bool {
	return typepublickey.W3IDSecurityV1PublicKeyExtends(other)
}
